@@ -1,46 +1,35 @@
-/********************************************************************
-Copyright 2013  Martin Gräßlin <mgraesslin@kde.org>
+/*
+    SPDX-FileCopyrightText: 2013 Martin Gräßlin <mgraesslin@kde.org>
 
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Lesser General Public
-License as published by the Free Software Foundation; either
-version 2.1 of the License, or (at your option) version 3, or any
-later version accepted by the membership of KDE e.V. (or its
-successor approved by the membership of KDE e.V.), which shall
-act as a proxy defined in Section 6 of version 3 of the license.
-
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public
-License along with this library.  If not, see <http://www.gnu.org/licenses/>.
-*********************************************************************/
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+*/
 #include "output.h"
 #include "wayland_pointer_p.h"
 // Qt
 #include <QPoint>
 #include <QRect>
-#include <QSize>
+#include <QVector>
 // wayland
 #include <wayland-client-protocol.h>
 
 namespace KWayland
 {
-
 namespace Client
 {
-
+namespace
+{
 typedef QList<Output::Mode> Modes;
+}
 
-class Output::Private
+class Q_DECL_HIDDEN Output::Private
 {
 public:
     Private(Output *q);
+    ~Private();
     void setup(wl_output *o);
 
-    WaylandPointer<wl_output, wl_output_destroy> output;
+    WaylandPointer<wl_output, wl_output_release> output;
+    EventQueue *queue = nullptr;
     QSize physicalSize;
     QPoint globalPosition;
     QString manufacturer;
@@ -50,14 +39,27 @@ public:
     Transform transform = Transform::Normal;
     Modes modes;
     Modes::iterator currentMode = modes.end();
+    QString name;
+    QString description;
+
+    static Output *get(wl_output *o);
 
 private:
-    static void geometryCallback(void *data, wl_output *output, int32_t x, int32_t y,
-                                 int32_t physicalWidth, int32_t physicalHeight, int32_t subPixel,
-                                 const char *make, const char *model, int32_t transform);
+    static void geometryCallback(void *data,
+                                 wl_output *output,
+                                 int32_t x,
+                                 int32_t y,
+                                 int32_t physicalWidth,
+                                 int32_t physicalHeight,
+                                 int32_t subPixel,
+                                 const char *make,
+                                 const char *model,
+                                 int32_t transform);
     static void modeCallback(void *data, wl_output *output, uint32_t flags, int32_t width, int32_t height, int32_t refresh);
     static void doneCallback(void *data, wl_output *output);
     static void scaleCallback(void *data, wl_output *output, int32_t scale);
+    static void nameCallback(void *data, struct wl_output *wl_output, const char *name);
+    static void descriptionCallback(void *data, struct wl_output *wl_output, const char *description);
     void setPhysicalSize(const QSize &size);
     void setGlobalPosition(const QPoint &pos);
     void setManufacturer(const QString &manufacturer);
@@ -69,11 +71,33 @@ private:
 
     Output *q;
     static struct wl_output_listener s_outputListener;
+
+    static QVector<Private *> s_allOutputs;
 };
 
+QVector<Output::Private *> Output::Private::s_allOutputs;
+
 Output::Private::Private(Output *q)
     : q(q)
 {
+    s_allOutputs << this;
+}
+
+Output::Private::~Private()
+{
+    s_allOutputs.removeOne(this);
+}
+
+Output *Output::Private::get(wl_output *o)
+{
+    auto it = std::find_if(s_allOutputs.constBegin(), s_allOutputs.constEnd(), [o](Private *p) {
+        const wl_output *reference = p->output;
+        return reference == o;
+    });
+    if (it != s_allOutputs.constEnd()) {
+        return (*it)->q;
+    }
+    return nullptr;
 }
 
 void Output::Private::setup(wl_output *o)
@@ -86,10 +110,7 @@ void Output::Private::setup(wl_output *o)
 
 bool Output::Mode::operator==(const Output::Mode &m) const
 {
-    return size == m.size
-           && refreshRate == m.refreshRate
-           && flags == m.flags
-           && output == m.output;
+    return size == m.size && refreshRate == m.refreshRate && flags == m.flags && output == m.output;
 }
 
 Output::Output(QObject *parent)
@@ -104,19 +125,28 @@ Output::~Output()
 }
 
 wl_output_listener Output::Private::s_outputListener = {
-    geometryCallback,
-    modeCallback,
-    doneCallback,
-    scaleCallback
+    geometryCallback, modeCallback, doneCallback, scaleCallback,
+#ifdef WL_OUTPUT_NAME_SINCE_VERSION
+    nameCallback,
+#endif
+#ifdef WL_OUTPUT_DESCRIPTION_SINCE_VERSION
+    descriptionCallback,
+#endif
 };
 
-void Output::Private::geometryCallback(void *data, wl_output *output,
-                              int32_t x, int32_t y,
-                              int32_t physicalWidth, int32_t physicalHeight,
-                              int32_t subPixel, const char *make, const char *model, int32_t transform)
+void Output::Private::geometryCallback(void *data,
+                                       wl_output *output,
+                                       int32_t x,
+                                       int32_t y,
+                                       int32_t physicalWidth,
+                                       int32_t physicalHeight,
+                                       int32_t subPixel,
+                                       const char *make,
+                                       const char *model,
+                                       int32_t transform)
 {
     Q_UNUSED(transform)
-    auto o = reinterpret_cast<Output::Private*>(data);
+    auto o = reinterpret_cast<Output::Private *>(data);
     Q_ASSERT(o->output == output);
     o->setGlobalPosition(QPoint(x, y));
     o->setManufacturer(make);
@@ -166,7 +196,7 @@ void Output::Private::geometryCallback(void *data, wl_output *output,
 
 void Output::Private::modeCallback(void *data, wl_output *output, uint32_t flags, int32_t width, int32_t height, int32_t refresh)
 {
-    auto o = reinterpret_cast<Output::Private*>(data);
+    auto o = reinterpret_cast<Output::Private *>(data);
     Q_ASSERT(o->output == output);
     o->addMode(flags, width, height, refresh);
 }
@@ -191,7 +221,7 @@ void Output::Private::addMode(uint32_t flags, int32_t width, int32_t height, int
             auto &m = (*it);
             if (m.flags.testFlag(Mode::Flag::Current)) {
                 m.flags &= ~Mode::Flags(Mode::Flag::Current);
-                emit q->modeChanged(m);
+                Q_EMIT q->modeChanged(m);
             }
             if (m.refreshRate == mode.refreshRate && m.size == mode.size) {
                 it = modes.erase(it);
@@ -203,24 +233,38 @@ void Output::Private::addMode(uint32_t flags, int32_t width, int32_t height, int
         currentMode = currentIt;
     }
     if (existing) {
-        emit q->modeChanged(mode);
+        Q_EMIT q->modeChanged(mode);
     } else {
-        emit q->modeAdded(mode);
+        Q_EMIT q->modeAdded(mode);
     }
 }
 
 void Output::Private::scaleCallback(void *data, wl_output *output, int32_t scale)
 {
-    auto o = reinterpret_cast<Output::Private*>(data);
+    auto o = reinterpret_cast<Output::Private *>(data);
     Q_ASSERT(o->output == output);
     o->setScale(scale);
 }
 
+void Output::Private::nameCallback(void *data, struct wl_output *wl_output, const char *name)
+{
+    auto o = reinterpret_cast<Output::Private *>(data);
+    Q_ASSERT(o->output == wl_output);
+    o->name = QString::fromUtf8(name);
+}
+
+void Output::Private::descriptionCallback(void *data, struct wl_output *wl_output, const char *description)
+{
+    auto o = reinterpret_cast<Output::Private *>(data);
+    Q_ASSERT(o->output == wl_output);
+    o->description = QString::fromUtf8(description);
+}
+
 void Output::Private::doneCallback(void *data, wl_output *output)
 {
-    auto o = reinterpret_cast<Output::Private*>(data);
+    auto o = reinterpret_cast<Output::Private *>(data);
     Q_ASSERT(o->output == output);
-    emit o->q->changed();
+    Q_EMIT o->q->changed();
 }
 
 void Output::setup(wl_output *output)
@@ -228,6 +272,16 @@ void Output::setup(wl_output *output)
     d->setup(output);
 }
 
+EventQueue *Output::eventQueue() const
+{
+    return d->queue;
+}
+
+void Output::setEventQueue(EventQueue *queue)
+{
+    d->queue = queue;
+}
+
 void Output::Private::setGlobalPosition(const QPoint &pos)
 {
     globalPosition = pos;
@@ -332,18 +386,40 @@ Output::Transform Output::transform() const
     return d->transform;
 }
 
-QList< Output::Mode > Output::modes() const
+QList<Output::Mode> Output::modes() const
 {
     return d->modes;
 }
 
-Output::operator wl_output*() {
+QString Output::name() const
+{
+    return d->name;
+}
+
+QString Output::description() const
+{
+    return d->description;
+}
+
+Output::operator wl_output *()
+{
     return d->output;
 }
 
-Output::operator wl_output*() const {
+Output::operator wl_output *() const
+{
     return d->output;
 }
 
+Output *Output::get(wl_output *o)
+{
+    return Private::get(o);
+}
+
+void Output::destroy()
+{
+    d->output.destroy();
+}
+
 }
 }
