@@ -1,36 +1,21 @@
-/********************************************************************
-Copyright 2014  Martin Gräßlin <mgraesslin@kde.org>
+/*
+    SPDX-FileCopyrightText: 2014 Martin Gräßlin <mgraesslin@kde.org>
 
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Lesser General Public
-License as published by the Free Software Foundation; either
-version 2.1 of the License, or (at your option) version 3, or any
-later version accepted by the membership of KDE e.V. (or its
-successor approved by the membership of KDE e.V.), which shall
-act as a proxy defined in Section 6 of version 3 of the license.
-
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public
-License along with this library.  If not, see <http://www.gnu.org/licenses/>.
-*********************************************************************/
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+*/
 #include "outputdevice_interface.h"
-#include "global_p.h"
 #include "display.h"
-#include "logging_p.h"
+#include "global_p.h"
+#include "logging.h"
 
-#include <wayland-server.h>
 #include "wayland-org_kde_kwin_outputdevice-server-protocol.h"
 #include <QDebug>
+#include <wayland-server.h>
 
 namespace KWayland
 {
 namespace Server
 {
-
 class OutputDeviceInterface::Private : public Global::Private
 {
 public:
@@ -39,25 +24,40 @@ public:
         uint32_t version;
     };
     Private(OutputDeviceInterface *q, Display *d);
-    ~Private();
+    ~Private() override;
 
+    void updateGeometry();
+    void updateUuid();
+    void updateEdid();
+    void updateEnabled();
+    void updateScale();
+    void updateColorCurves();
+    void updateEisaId();
+    void updateSerialNumber();
+
+    void sendGeometry(wl_resource *resource);
     void sendMode(wl_resource *resource, const Mode &mode);
     void sendDone(const ResourceData &data);
-    void updateGeometry();
-    void updateScale();
-
-    void sendUuid();
-    void sendEdid();
-    void sendEnabled();
+    void sendUuid(const ResourceData &data);
+    void sendEdid(const ResourceData &data);
+    void sendEnabled(const ResourceData &data);
+    void sendScale(const ResourceData &data);
+    void sendColorCurves(const ResourceData &data);
+    void sendEisaId(const ResourceData &data);
+    void sendSerialNumber(const ResourceData &data);
 
     QSize physicalSize;
     QPoint globalPosition;
     QString manufacturer = QStringLiteral("org.kde.kwin");
     QString model = QStringLiteral("none");
-    int scale = 1;
+    qreal scale = 1.0;
+    QString serialNumber;
+    QString eisaId;
     SubPixel subPixel = SubPixel::Unknown;
     Transform transform = Transform::Normal;
+    ColorCurves colorCurves;
     QList<Mode> modes;
+    Mode currentMode;
     QList<ResourceData> resources;
 
     QByteArray edid;
@@ -72,17 +72,15 @@ private:
     void bind(wl_client *client, uint32_t version, uint32_t id) override;
     int32_t toTransform() const;
     int32_t toSubPixel() const;
-    void sendGeometry(wl_resource *resource);
-    void sendScale(const ResourceData &data);
 
     static const quint32 s_version;
     OutputDeviceInterface *q;
-    static QVector<Private*> s_privates;
+    static QVector<Private *> s_privates;
 };
 
-const quint32 OutputDeviceInterface::Private::s_version = 1;
+const quint32 OutputDeviceInterface::Private::s_version = 2;
 
-QVector<OutputDeviceInterface::Private*> OutputDeviceInterface::Private::s_privates;
+QVector<OutputDeviceInterface::Private *> OutputDeviceInterface::Private::s_privates;
 
 OutputDeviceInterface::Private::Private(OutputDeviceInterface *q, Display *d)
     : Global::Private(d, &org_kde_kwin_outputdevice_interface, s_version)
@@ -108,7 +106,9 @@ OutputDeviceInterface::Private *OutputDeviceInterface::Private::cast(wl_resource
 {
     for (auto it = s_privates.constBegin(); it != s_privates.constEnd(); ++it) {
         const auto &resources = (*it)->resources;
-        auto rit = std::find_if(resources.begin(), resources.end(), [native] (const ResourceData &data) { return data.resource == native; });
+        auto rit = std::find_if(resources.begin(), resources.end(), [native](const ResourceData &data) {
+            return data.resource == native;
+        });
         if (rit != resources.end()) {
             return (*it);
         }
@@ -120,25 +120,38 @@ OutputDeviceInterface::OutputDeviceInterface(Display *display, QObject *parent)
     : Global(new Private(this, display), parent)
 {
     Q_D();
-    connect(this, &OutputDeviceInterface::currentModeChanged, this,
-        [this, d] {
-            auto currentModeIt = std::find_if(d->modes.constBegin(), d->modes.constEnd(), [](const Mode &mode) { return mode.flags.testFlag(ModeFlag::Current); });
-            if (currentModeIt == d->modes.constEnd()) {
-                return;
-            }
-            for (auto it = d->resources.constBegin(); it != d->resources.constEnd(); ++it) {
-                d->sendMode((*it).resource, *currentModeIt);
-                d->sendDone(*it);
-            }
-            wl_display_flush_clients(*(d->display));
+    connect(this, &OutputDeviceInterface::currentModeChanged, this, [d] {
+        Q_ASSERT(d->currentMode.id >= 0);
+        for (auto it = d->resources.constBegin(); it != d->resources.constEnd(); ++it) {
+            d->sendMode((*it).resource, d->currentMode);
+            d->sendDone(*it);
         }
-    );
-    connect(this, &OutputDeviceInterface::subPixelChanged,       this, [this, d] { d->updateGeometry(); });
-    connect(this, &OutputDeviceInterface::transformChanged,      this, [this, d] { d->updateGeometry(); });
-    connect(this, &OutputDeviceInterface::globalPositionChanged, this, [this, d] { d->updateGeometry(); });
-    connect(this, &OutputDeviceInterface::modelChanged,          this, [this, d] { d->updateGeometry(); });
-    connect(this, &OutputDeviceInterface::manufacturerChanged,   this, [this, d] { d->updateGeometry(); });
-    connect(this, &OutputDeviceInterface::scaleChanged,          this, [this, d] { d->updateScale(); });
+        wl_display_flush_clients(*(d->display));
+    });
+    connect(this, &OutputDeviceInterface::subPixelChanged, this, [d] {
+        d->updateGeometry();
+    });
+    connect(this, &OutputDeviceInterface::transformChanged, this, [d] {
+        d->updateGeometry();
+    });
+    connect(this, &OutputDeviceInterface::globalPositionChanged, this, [d] {
+        d->updateGeometry();
+    });
+    connect(this, &OutputDeviceInterface::modelChanged, this, [d] {
+        d->updateGeometry();
+    });
+    connect(this, &OutputDeviceInterface::manufacturerChanged, this, [d] {
+        d->updateGeometry();
+    });
+    connect(this, &OutputDeviceInterface::scaleFChanged, this, [d] {
+        d->updateScale();
+    });
+    connect(this, &OutputDeviceInterface::scaleChanged, this, [d] {
+        d->updateScale();
+    });
+    connect(this, &OutputDeviceInterface::colorCurvesChanged, this, [d] {
+        d->updateColorCurves();
+    });
 }
 
 OutputDeviceInterface::~OutputDeviceInterface() = default;
@@ -146,18 +159,14 @@ OutputDeviceInterface::~OutputDeviceInterface() = default;
 QSize OutputDeviceInterface::pixelSize() const
 {
     Q_D();
-    auto it = std::find_if(d->modes.begin(), d->modes.end(),
-        [](const Mode &mode) {
-            return mode.flags.testFlag(ModeFlag::Current);
-        }
-    );
-    if (it == d->modes.end()) {
+
+    if (d->currentMode.id == -1) {
         return QSize();
     }
-    return (*it).size;
+    return d->currentMode.size;
 }
 
-OutputDeviceInterface *OutputDeviceInterface::get(wl_resource* native)
+OutputDeviceInterface *OutputDeviceInterface::get(wl_resource *native)
 {
     return Private::get(native);
 }
@@ -165,28 +174,23 @@ OutputDeviceInterface *OutputDeviceInterface::get(wl_resource* native)
 int OutputDeviceInterface::refreshRate() const
 {
     Q_D();
-    auto it = std::find_if(d->modes.begin(), d->modes.end(),
-        [](const Mode &mode) {
-            return mode.flags.testFlag(ModeFlag::Current);
-        }
-    );
-    if (it == d->modes.end()) {
+
+    if (d->currentMode.id == -1) {
         return 60000;
     }
-    return (*it).refreshRate;
+    return d->currentMode.refreshRate;
 }
 
 void OutputDeviceInterface::addMode(Mode &mode)
 {
     Q_ASSERT(!isValid());
+    Q_ASSERT(mode.id >= 0);
+    Q_ASSERT(mode.size.isValid());
     Q_D();
 
-
-    auto currentModeIt = std::find_if(d->modes.begin(), d->modes.end(),
-        [](const Mode &mode) {
-            return mode.flags.testFlag(ModeFlag::Current);
-        }
-    );
+    auto currentModeIt = std::find_if(d->modes.begin(), d->modes.end(), [](const Mode &mode) {
+        return mode.flags.testFlag(ModeFlag::Current);
+    });
     if (currentModeIt == d->modes.end() && !mode.flags.testFlag(ModeFlag::Current)) {
         // no mode with current flag - enforce
         mode.flags |= ModeFlag::Current;
@@ -198,29 +202,24 @@ void OutputDeviceInterface::addMode(Mode &mode)
 
     if (mode.flags.testFlag(ModeFlag::Preferred)) {
         // remove from existing Preferred mode
-        auto preferredIt = std::find_if(d->modes.begin(), d->modes.end(),
-            [](const Mode &mode) {
-                return mode.flags.testFlag(ModeFlag::Preferred);
-            }
-        );
+        auto preferredIt = std::find_if(d->modes.begin(), d->modes.end(), [](const Mode &mode) {
+            return mode.flags.testFlag(ModeFlag::Preferred);
+        });
         if (preferredIt != d->modes.end()) {
             (*preferredIt).flags &= ~uint(ModeFlag::Preferred);
         }
     }
 
-    auto existingModeIt = std::find_if(d->modes.begin(), d->modes.end(),
-        [mode](const Mode &mode_it) {
-            return mode.size == mode_it.size &&
-                   mode.refreshRate == mode_it.refreshRate &&
-                   mode.id == mode_it.id;
-        }
-    );
-    auto emitChanges = [this,mode] {
-        emit modesChanged();
+    auto existingModeIt = std::find_if(d->modes.begin(), d->modes.end(), [mode](const Mode &mode_it) {
+        return mode.size == mode_it.size && mode.refreshRate == mode_it.refreshRate && mode.id == mode_it.id;
+    });
+    auto emitChanges = [this, d, mode] {
+        Q_EMIT modesChanged();
         if (mode.flags.testFlag(ModeFlag::Current)) {
-            emit refreshRateChanged(mode.refreshRate);
-            emit pixelSizeChanged(mode.size);
-            emit currentModeChanged();
+            d->currentMode = mode;
+            Q_EMIT refreshRateChanged(mode.refreshRate);
+            Q_EMIT pixelSizeChanged(mode.size);
+            Q_EMIT currentModeChanged();
         }
     };
     if (existingModeIt != d->modes.end()) {
@@ -232,16 +231,13 @@ void OutputDeviceInterface::addMode(Mode &mode)
         emitChanges();
         return;
     } else {
-        auto idIt = std::find_if(d->modes.begin(), d->modes.end(),
-                                        [mode](const Mode &mode_it) {
-                                            return mode.id == mode_it.id;
-                                        }
-        );
-        if (idIt != d->modes.end()) {
+        auto idIt = std::find_if(d->modes.constBegin(), d->modes.constEnd(), [mode](const Mode &mode_it) {
+            return mode.id == mode_it.id;
+        });
+        if (idIt != d->modes.constEnd()) {
             qCWarning(KWAYLAND_SERVER) << "Duplicate Mode id" << mode.id << ": not adding mode" << mode.size << mode.refreshRate;
             return;
         }
-
     }
     d->modes << mode;
     emitChanges();
@@ -250,28 +246,25 @@ void OutputDeviceInterface::addMode(Mode &mode)
 void OutputDeviceInterface::setCurrentMode(const int modeId)
 {
     Q_D();
-    auto currentModeIt = std::find_if(d->modes.begin(), d->modes.end(),
-        [](const Mode &mode) {
-            return mode.flags.testFlag(ModeFlag::Current);
-        }
-    );
+    auto currentModeIt = std::find_if(d->modes.begin(), d->modes.end(), [](const Mode &mode) {
+        return mode.flags.testFlag(ModeFlag::Current);
+    });
     if (currentModeIt != d->modes.end()) {
         // another mode has the current flag - remove
         (*currentModeIt).flags &= ~uint(ModeFlag::Current);
     }
 
-    auto existingModeIt = std::find_if(d->modes.begin(), d->modes.end(),
-        [modeId](const Mode &mode) {
-            return mode.id == modeId;
-        }
-    );
+    auto existingModeIt = std::find_if(d->modes.begin(), d->modes.end(), [modeId](const Mode &mode) {
+        return mode.id == modeId;
+    });
 
     Q_ASSERT(existingModeIt != d->modes.end());
     (*existingModeIt).flags |= ModeFlag::Current;
-    emit modesChanged();
-    emit refreshRateChanged((*existingModeIt).refreshRate);
-    emit pixelSizeChanged((*existingModeIt).size);
-    emit currentModeChanged();
+    d->currentMode = *existingModeIt;
+    Q_EMIT modesChanged();
+    Q_EMIT refreshRateChanged((*existingModeIt).refreshRate);
+    Q_EMIT pixelSizeChanged((*existingModeIt).size);
+    Q_EMIT currentModeChanged();
 }
 
 int32_t OutputDeviceInterface::Private::toTransform() const
@@ -333,6 +326,9 @@ void OutputDeviceInterface::Private::bind(wl_client *client, uint32_t version, u
 
     sendGeometry(resource);
     sendScale(r);
+    sendColorCurves(r);
+    sendEisaId(r);
+    sendSerialNumber(r);
 
     auto currentModeIt = modes.constEnd();
     for (auto it = modes.constBegin(); it != modes.constEnd(); ++it) {
@@ -349,9 +345,9 @@ void OutputDeviceInterface::Private::bind(wl_client *client, uint32_t version, u
         sendMode(resource, *currentModeIt);
     }
 
-    sendUuid();
-    sendEdid();
-    sendEnabled();
+    sendUuid(r);
+    sendEdid(r);
+    sendEnabled(r);
 
     sendDone(r);
     c->flush();
@@ -363,7 +359,9 @@ void OutputDeviceInterface::Private::unbind(wl_resource *resource)
     if (!o) {
         return;
     }
-    auto it = std::find_if(o->resources.begin(), o->resources.end(), [resource](const ResourceData &r) { return r.resource == resource; });
+    auto it = std::find_if(o->resources.begin(), o->resources.end(), [resource](const ResourceData &r) {
+        return r.resource == resource;
+    });
     if (it != o->resources.end()) {
         o->resources.erase(it);
     }
@@ -378,31 +376,70 @@ void OutputDeviceInterface::Private::sendMode(wl_resource *resource, const Mode
     if (mode.flags.testFlag(ModeFlag::Preferred)) {
         flags |= WL_OUTPUT_MODE_PREFERRED;
     }
-    org_kde_kwin_outputdevice_send_mode(resource,
-                        flags,
-                        mode.size.width(),
-                        mode.size.height(),
-                        mode.refreshRate,
-                        mode.id);
-
+    org_kde_kwin_outputdevice_send_mode(resource, flags, mode.size.width(), mode.size.height(), mode.refreshRate, mode.id);
 }
 
 void OutputDeviceInterface::Private::sendGeometry(wl_resource *resource)
 {
     org_kde_kwin_outputdevice_send_geometry(resource,
-                            globalPosition.x(),
-                            globalPosition.y(),
-                            physicalSize.width(),
-                            physicalSize.height(),
-                            toSubPixel(),
-                            qPrintable(manufacturer),
-                            qPrintable(model),
-                            toTransform());
+                                            globalPosition.x(),
+                                            globalPosition.y(),
+                                            physicalSize.width(),
+                                            physicalSize.height(),
+                                            toSubPixel(),
+                                            qPrintable(manufacturer),
+                                            qPrintable(model),
+                                            toTransform());
 }
 
 void OutputDeviceInterface::Private::sendScale(const ResourceData &data)
 {
-    org_kde_kwin_outputdevice_send_scale(data.resource, scale);
+    if (wl_resource_get_version(data.resource) < ORG_KDE_KWIN_OUTPUTDEVICE_SCALEF_SINCE_VERSION) {
+        org_kde_kwin_outputdevice_send_scale(data.resource, qRound(scale));
+    } else {
+        org_kde_kwin_outputdevice_send_scalef(data.resource, wl_fixed_from_double(scale));
+    }
+}
+
+void OutputDeviceInterface::Private::sendColorCurves(const ResourceData &data)
+{
+    if (data.version < ORG_KDE_KWIN_OUTPUTDEVICE_COLORCURVES_SINCE_VERSION) {
+        return;
+    }
+
+    wl_array wlRed;
+    wl_array wlGreen;
+    wl_array wlBlue;
+
+    auto fillArray = [](const QVector<quint16> &origin, wl_array *dest) {
+        wl_array_init(dest);
+        const size_t memLength = sizeof(uint16_t) * origin.size();
+        void *s = wl_array_add(dest, memLength);
+        memcpy(s, origin.data(), memLength);
+    };
+    fillArray(colorCurves.red, &wlRed);
+    fillArray(colorCurves.green, &wlGreen);
+    fillArray(colorCurves.blue, &wlBlue);
+
+    org_kde_kwin_outputdevice_send_colorcurves(data.resource, &wlRed, &wlGreen, &wlBlue);
+
+    wl_array_release(&wlRed);
+    wl_array_release(&wlGreen);
+    wl_array_release(&wlBlue);
+}
+
+void KWayland::Server::OutputDeviceInterface::Private::sendSerialNumber(const ResourceData &data)
+{
+    if (wl_resource_get_version(data.resource) >= ORG_KDE_KWIN_OUTPUTDEVICE_SERIAL_NUMBER_SINCE_VERSION) {
+        org_kde_kwin_outputdevice_send_serial_number(data.resource, qPrintable(serialNumber));
+    }
+}
+
+void KWayland::Server::OutputDeviceInterface::Private::sendEisaId(const ResourceData &data)
+{
+    if (wl_resource_get_version(data.resource) >= ORG_KDE_KWIN_OUTPUTDEVICE_EISA_ID_SINCE_VERSION) {
+        org_kde_kwin_outputdevice_send_eisa_id(data.resource, qPrintable(eisaId));
+    }
 }
 
 void OutputDeviceInterface::Private::sendDone(const ResourceData &data)
@@ -426,6 +463,24 @@ void OutputDeviceInterface::Private::updateScale()
     }
 }
 
+void OutputDeviceInterface::Private::updateColorCurves()
+{
+    for (auto it = resources.constBegin(); it != resources.constEnd(); ++it) {
+        sendColorCurves(*it);
+        sendDone(*it);
+    }
+}
+
+bool OutputDeviceInterface::ColorCurves::operator==(const ColorCurves &cc) const
+{
+    return red == cc.red && green == cc.green && blue == cc.blue;
+}
+bool OutputDeviceInterface::ColorCurves::operator!=(const ColorCurves &cc) const
+{
+    return !operator==(cc);
+}
+
+// clang-format off
 #define SETTER(setterName, type, argumentName) \
     void OutputDeviceInterface::setterName(type arg) \
     { \
@@ -434,19 +489,43 @@ void OutputDeviceInterface::Private::updateScale()
             return; \
         } \
         d->argumentName = arg; \
-        emit argumentName##Changed(d->argumentName); \
+        Q_EMIT argumentName##Changed(d->argumentName); \
     }
+// clang-format on
 
-SETTER(setPhysicalSize, const QSize&, physicalSize)
-SETTER(setGlobalPosition, const QPoint&, globalPosition)
-SETTER(setManufacturer, const QString&, manufacturer)
-SETTER(setModel, const QString&, model)
-SETTER(setScale, int, scale)
+SETTER(setPhysicalSize, const QSize &, physicalSize)
+SETTER(setGlobalPosition, const QPoint &, globalPosition)
+SETTER(setManufacturer, const QString &, manufacturer)
+SETTER(setModel, const QString &, model)
+SETTER(setSerialNumber, const QString &, serialNumber)
+SETTER(setEisaId, const QString &, eisaId)
 SETTER(setSubPixel, SubPixel, subPixel)
 SETTER(setTransform, Transform, transform)
 
 #undef SETTER
 
+void OutputDeviceInterface::setScale(int scale)
+{
+    Q_D();
+    if (d->scale == scale) {
+        return;
+    }
+    d->scale = scale;
+    Q_EMIT scaleChanged(d->scale);
+    Q_EMIT scaleFChanged(d->scale);
+}
+
+void OutputDeviceInterface::setScaleF(qreal scale)
+{
+    Q_D();
+    if (qFuzzyCompare(d->scale, scale)) {
+        return;
+    }
+    d->scale = scale;
+    Q_EMIT scaleChanged(qRound(d->scale));
+    Q_EMIT scaleFChanged(d->scale);
+}
+
 QSize OutputDeviceInterface::physicalSize() const
 {
     Q_D();
@@ -471,7 +550,25 @@ QString OutputDeviceInterface::model() const
     return d->model;
 }
 
+QString OutputDeviceInterface::serialNumber() const
+{
+    Q_D();
+    return d->serialNumber;
+}
+
+QString OutputDeviceInterface::eisaId() const
+{
+    Q_D();
+    return d->eisaId;
+}
+
 int OutputDeviceInterface::scale() const
+{
+    Q_D();
+    return qRound(d->scale);
+}
+
+qreal OutputDeviceInterface::scaleF() const
 {
     Q_D();
     return d->scale;
@@ -489,7 +586,13 @@ OutputDeviceInterface::Transform OutputDeviceInterface::transform() const
     return d->transform;
 }
 
-QList< OutputDeviceInterface::Mode > OutputDeviceInterface::modes() const
+OutputDeviceInterface::ColorCurves OutputDeviceInterface::colorCurves() const
+{
+    Q_D();
+    return d->colorCurves;
+}
+
+QList<OutputDeviceInterface::Mode> OutputDeviceInterface::modes() const
 {
     Q_D();
     return d->modes;
@@ -498,7 +601,7 @@ QList< OutputDeviceInterface::Mode > OutputDeviceInterface::modes() const
 int OutputDeviceInterface::currentModeId() const
 {
     Q_D();
-    for (const Mode &m: d->modes) {
+    for (const Mode &m : d->modes) {
         if (m.flags.testFlag(OutputDeviceInterface::ModeFlag::Current)) {
             return m.id;
         }
@@ -508,15 +611,26 @@ int OutputDeviceInterface::currentModeId() const
 
 OutputDeviceInterface::Private *OutputDeviceInterface::d_func() const
 {
-    return reinterpret_cast<Private*>(d.data());
+    return reinterpret_cast<Private *>(d.data());
+}
+
+void OutputDeviceInterface::setColorCurves(const ColorCurves &colorCurves)
+{
+    Q_D();
+
+    if (d->colorCurves == colorCurves) {
+        return;
+    }
+    d->colorCurves = colorCurves;
+    Q_EMIT colorCurvesChanged(d->colorCurves);
 }
 
 void OutputDeviceInterface::setEdid(const QByteArray &edid)
 {
     Q_D();
     d->edid = edid;
-    d->sendEdid();
-    emit edidChanged();
+    d->updateEdid();
+    Q_EMIT edidChanged();
 }
 
 QByteArray OutputDeviceInterface::edid() const
@@ -530,8 +644,8 @@ void OutputDeviceInterface::setEnabled(OutputDeviceInterface::Enablement enabled
     Q_D();
     if (d->enabled != enabled) {
         d->enabled = enabled;
-        d->sendEnabled();
-        emit enabledChanged();
+        d->updateEnabled();
+        Q_EMIT enabledChanged();
     }
 }
 
@@ -546,8 +660,8 @@ void OutputDeviceInterface::setUuid(const QByteArray &uuid)
     Q_D();
     if (d->uuid != uuid) {
         d->uuid = uuid;
-        d->sendUuid();
-        emit uuidChanged();
+        d->updateUuid();
+        Q_EMIT uuidChanged();
     }
 }
 
@@ -557,30 +671,50 @@ QByteArray OutputDeviceInterface::uuid() const
     return d->uuid;
 }
 
-void KWayland::Server::OutputDeviceInterface::Private::sendEdid()
+void KWayland::Server::OutputDeviceInterface::Private::sendEdid(const ResourceData &data)
 {
-    for (auto it = resources.constBegin(); it != resources.constEnd(); ++it) {
-        org_kde_kwin_outputdevice_send_edid((*it).resource,
-                                            edid.toBase64().constData());
-    }
-
+    org_kde_kwin_outputdevice_send_edid(data.resource, edid.toBase64().constData());
 }
 
-void KWayland::Server::OutputDeviceInterface::Private::sendEnabled()
+void KWayland::Server::OutputDeviceInterface::Private::sendEnabled(const ResourceData &data)
 {
     int _enabled = 0;
     if (enabled == OutputDeviceInterface::Enablement::Enabled) {
         _enabled = 1;
     }
-    for (auto it = resources.constBegin(); it != resources.constEnd(); ++it) {
-        org_kde_kwin_outputdevice_send_enabled((*it).resource, _enabled);
-    }
+    org_kde_kwin_outputdevice_send_enabled(data.resource, _enabled);
 }
 
-void OutputDeviceInterface::Private::sendUuid()
+void OutputDeviceInterface::Private::sendUuid(const ResourceData &data)
+{
+    org_kde_kwin_outputdevice_send_uuid(data.resource, uuid.constData());
+}
+
+void KWayland::Server::OutputDeviceInterface::Private::updateEnabled()
 {
     for (auto it = resources.constBegin(); it != resources.constEnd(); ++it) {
-        org_kde_kwin_outputdevice_send_uuid((*it).resource, uuid.constData());
+        sendEnabled(*it);
+    }
+}
+
+void KWayland::Server::OutputDeviceInterface::Private::updateEdid()
+{
+    for (auto it = resources.constBegin(); it != resources.constEnd(); ++it) {
+        sendEdid(*it);
+    }
+}
+
+void KWayland::Server::OutputDeviceInterface::Private::updateUuid()
+{
+    for (auto it = resources.constBegin(); it != resources.constEnd(); ++it) {
+        sendUuid(*it);
+    }
+}
+
+void KWayland::Server::OutputDeviceInterface::Private::updateEisaId()
+{
+    for (auto it = resources.constBegin(); it != resources.constEnd(); ++it) {
+        sendEisaId(*it);
     }
 }
 
